# Copyright 2005 PyTom <pytom@bishoujo.us>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import pickle
import alice

class Sizes(object):

    def __init__(self, size):
        self.scale = size
        self.crop = size
        self.offset = 0

class Recolor(object):

    def __init__(self,
                 h_offset=0.0,
                 s_multiply=1.0,
                 s_offset=0.0,
                 l_multiply=1.0,
                 l_offset=0.0):

        self.h_offset = h_offset
        self.s_multiply = s_multiply
        self.s_offset = s_offset
        self.l_multiply = l_multiply
        self.l_offset = l_offset

    def data_tuple(self):
        return self.h_offset, self.s_multiply, self.s_offset, self.l_multiply, self.l_offset
        
    def __eq__(self, o):
        return self.data_tuple() == o.data_tuple()

    def __hash__(self):
        return hash(self.data_tuple())
        


class State(object):
    """
    This file contains the session state that is persisted out to disk.
    """

    def __setstate__(self, state):

        if "recolors" not in state:
            state["recolors"] = { }

        self.__dict__.update(state)
        

    def __init__(self, doll):

        # The name of the doll this state uses.
        self.doll = doll

        # The sizes in h and v.
        d = alice.dolls.dolls[doll]
        width, height = d.sizes()

        self.h = Sizes(width)
        self.v = Sizes(height)

        # A map from configuration name to a list of the layers that
        # are used in that configuration. Layers are represented as
        # category, layer tuples.
        self.configs = { }
        self.configs['untitled'] = d.config.get('default', [ ])

        # Fixup the sizes.
        self.h.scale = d.config.get('hscale', width)
        self.h.crop = d.config.get('hcrop', self.h.scale)
        self.h.offset = d.config.get('hoffset', 0)

        self.v.scale = d.config.get('vscale', height)
        self.v.crop = d.config.get('vcrop', self.v.scale)
        self.v.offset = d.config.get('voffset', 0)

        # A map from (category, layer, image) pairs to Recolors.
        self.recolors = { }

        # The filename this state corresponds to. This is set when it is
        # loaded, and changed when it is saved.
        self.filename = None

        # The directory we will render into.
        self.renderdir = None

    

    def store(self):
        f = file(self.filename, "wb")
        pickle.dump(self, f, protocol=2)
        f.close()
        

def load(fn):

    f = file(fn, "rb")
    rv = pickle.load(f)
    f.close()

    rv.filename = fn

    if rv.doll not in alice.dolls.dolls:
        raise Exception(fn + " uses doll " + rv.doll + ", which we do not know about.")
    
    return rv
    

        
