#include "alice.h"

/*
 * This was taken from:
 *
 * http://groups.google.com/group/comp.graphics/msg/8f909ce27f47b451?hl=en&
 *
 */

/*
A Fast HSL-to-RGB Transform
by Ken Fishkin
from "Graphics Gems", Academic Press, 1990
*/

#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

    /*
     * RGB-HSL transforms.
     * Ken Fishkin, Pixar Inc., January 1989.
     */

    /*
    * given r,g,b on [0 ... 1],
    * return (h,s,l) on [0 ... 1]
    */

void inline color_single(
    unsigned char *rp,
    unsigned char *gp,
    unsigned char *bp,
    float hoff, /* Must be in [0, 6] */
    float smul, 
    float soff,
    float lmul,
    float loff
    ) {

    float r;
    float g;
    float b;

    float h;
    float s;
    float l;

    float v;
    float m;
    float vm;
    float r2, g2, b2;

    float sv;
    int sextant;
    float fract, vsf, mid1, mid2;

    r = *rp * (1.0 / 255.0);
    g = *gp * (1.0 / 255.0);
    b = *bp * (1.0 / 255.0);
    
    v = MAX(r,g);
    v = MAX(v,b);
    m = MIN(r,g);
    m = MIN(m,b);

    if ((l = (m + v) * .5) <= 0.0) {
        h = 0.0;
        s = 0.0;
        l = 0.0;
    } else if ((s = vm = v - m) > 0.0) {
        s /= (l <= 0.5) ? (v + m ) :
            (2.0 - v - m) ;

        r2 = (v - r) / vm;
        g2 = (v - g) / vm;
        b2 = (v - b) / vm;

        if (r == v)
            h = (g == m ? 5.0 + b2 : 1.0 - g2);
        else if (g == v)
            h = (b == m ? 1.0 + r2 : 3.0 - b2);
        else
            h = (r == m ? 3.0 + g2 : 5.0 - r2);

    } else {
        h = 0.0;
        s = 0.0;
    }



    /* At this point, HSL are correct, with H in the range 0-6, since
       we eliminated a downscaling. */

    h += hoff;

    if (h >= 6.0) {
        h -= 6.0;
    }

    s = s * smul + soff;
    if (s < 0) {
        s = 0;
    } else if (s > 1) {
        s = 1;
    }

    l = l * lmul + loff;
    if (l < 0) {
        l = 0;
    } else if (l > 1) {
        l = 1;
    }
        
    /* At this point, HSL have been changed, and we're ready to begin
       converting back. */

    v = (l <= 0.5) ? (l * (1.0 + s)) : (l + s - l * s);
    if (v <= 0) {
        r = g = b = 0.0;
    } else {

        m = l + l - v;
        sv = (v - m ) / v;
        sextant = h;    
        fract = h - sextant;
        vsf = v * sv * fract;
        mid1 = m + vsf;
        mid2 = v - vsf;
        switch (sextant) {
        case 0: r = v; g = mid1; b = m; break;
        case 1: r = mid2; g = v; b = m; break;
        case 2: r = m; g = v; b = mid1; break;
        case 3: r = m; g = mid2; b = v; break;
        case 4: r = mid1; g = m; b = v; break;
        case 5: r = v; g = m; b = mid2; break;
        }
    }

    *rp = r * 255.0;
    *gp = g * 255.0;
    *bp = b * 255.0;
}


void color_image(
    struct ImagingMemoryInstance *im,
    float hoff,
    float smul,
    float soff,
    float lmul,
    float loff
    ) {
    int x;
    int y;

    unsigned char *p;

    hoff *= 6;
    
    for (y = 0; y < im->ysize; y++) {
        p = (unsigned char *) im->image[y];

        for (x = 0; x < im->xsize; x++) {
            color_single(p, p + 1, p + 2, hoff, smul, soff, lmul, loff);
            p += 4;
        }
    }
}
